/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.bo.message;

import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.COMPONENT;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.CONTAINER;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.CONTENT;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.ENDPOINT_NAME;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.ID;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.OPERATION;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.ROLE;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.SERVICE_NAME;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.STATUS;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.TIME;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.TYPE;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.MEP;
import static org.ow2.petals.tools.jmx.api.IPetalsMonitoringService.ReportKeyNames.EXCEPTION;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.hibernate.annotations.GenericGenerator;

/**
 * @author mcarpentier - EBM WebSourcing
 * 
 * A {@link MessageExchangeSnapshot} is the state of a Message Exchange at a
 * given time
 * 
 */
@Entity
public class MessageExchangeSnapshot implements Serializable {

	/**
	 * Serial UID
	 */
	private static final long serialVersionUID = -1434628041592824361L;

	private String messageExchangeSnapshotId;

	private long time;

	private long roundedTime;

	private String component;

	private String petalsId;

	private String role;

	private String status;

	private String content;

	private String service;

	private String endpoint;

	private String server;

	private String type;

	private String operation;

	private String mep;

	public MessageExchangeSnapshot() {
		super();
	}

	/**
	 * @param msg
	 *            a Map usually returned by a JMX call
	 */
	public MessageExchangeSnapshot(final Map<String, Object> msg) {
		super();
		if (msg.get(COMPONENT) != null) {
			this.setComponent((String) msg.get(COMPONENT));
		}
		if (msg.get(ID) != null) {
			this.setPetalsId((String) msg.get(ID));
		}
		if (msg.get(CONTAINER) != null) {
			this.setServer((String) msg.get(CONTAINER));
		}
		if (msg.get(EXCEPTION) != null) {
			this.setContent((String) msg.get(EXCEPTION));
		} else if (msg.get(CONTENT) != null) {
			this.setContent((String) msg.get(CONTENT));
		}
		if (msg.get(TIME) != null) {
			this.setTime(((Long) msg.get(TIME)).longValue());
			this.setRoundedTime((this.time / 1000) * 1000);
		}
		if (msg.get(ROLE) != null) {
			this.setRole((String) msg.get(ROLE));
		}
		if (msg.get(TYPE) != null) {
			this.setType((String) msg.get(TYPE));
		}
		if (msg.get(STATUS) != null) {
			this.setStatus((String) msg.get(STATUS));
		}
		if (msg.get(SERVICE_NAME) != null) {
			this.setService((String) msg.get(SERVICE_NAME));
		}
		if (msg.get(ENDPOINT_NAME) != null) {
			this.setEndpoint((String) msg.get(ENDPOINT_NAME));
		}
		if (msg.get(OPERATION) != null) {
			this.setOperation((String) msg.get(OPERATION));
		}
		if (msg.get(MEP) != null) {
			this.setMep((String) msg.get(MEP));
		}
	}

	/**
	 * generate a map with all properties of the {@link MessageExchangeSnapshot}
	 * useful for JMX calls
	 * 
	 * @return
	 */
	public Map<String, Object> generateObjectMap() {
		final Map<String, Object> result = new HashMap<String, Object>();
		result.put(COMPONENT, this.component);
		result.put(ID, this.petalsId);
		result.put(CONTAINER, this.server);
		result.put(CONTENT, this.content);
		result.put(TIME, this.time);
		result.put(ROLE, this.role);
		result.put(TYPE, this.type);
		result.put(STATUS, this.status);
		result.put(SERVICE_NAME, this.service);
		result.put(ENDPOINT_NAME, this.endpoint);
		result.put(OPERATION, this.operation);
		result.put(MEP, this.mep);
		return result;

	}

	public String getComponent() {
		return this.component;
	}

	public String getContent() {
		return this.content;
	}

	public String getEndpoint() {
		return this.endpoint;
	}

	@Id
	@GeneratedValue(generator = "system-uuid")
	@GenericGenerator(name = "system-uuid", strategy = "uuid")
	public String getMessageExchangeSnapshotId() {
		return this.messageExchangeSnapshotId;
	}

	public String getRole() {
		return this.role;
	}

	public String getService() {
		return this.service;
	}

	public String getStatus() {
		return this.status;
	}

	public long getTime() {
		return this.time;
	}

	public String getType() {
		return this.type;
	}

	public String getPetalsId() {
		return petalsId;
	}

	public void setPetalsId(String petalsId) {
		this.petalsId = petalsId;
	}

	public void setComponent(final String component) {
		this.component = component;
	}

	public void setContent(final String content) {
		this.content = content;
	}

	public void setEndpoint(final String endpoint) {
		this.endpoint = endpoint;
	}

	public void setMessageExchangeSnapshotId(
			final String messageExchangeSnapshotId) {
		this.messageExchangeSnapshotId = messageExchangeSnapshotId;
	}

	public void setRole(final String role) {
		this.role = role;
	}

	public void setService(final String service) {
		this.service = service;
	}

	public void setStatus(final String status) {
		this.status = status;
	}

	public void setTime(final long time) {
		this.time = time;
	}

	public void setType(final String type) {
		this.type = type;
	}

	public String getServer() {
		return server;
	}

	public void setServer(String server) {
		this.server = server;
	}

	public String getOperation() {
		return operation;
	}

	public void setOperation(String operation) {
		this.operation = operation;
	}

	public long getRoundedTime() {
		return roundedTime;
	}

	public void setRoundedTime(long roundedTime) {
		this.roundedTime = roundedTime;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object object) {
		if (!(object instanceof MessageExchangeSnapshot)) {
			return false;
		}
		MessageExchangeSnapshot rhs = (MessageExchangeSnapshot) object;
		return new EqualsBuilder().appendSuper(super.equals(object)).append(
				this.roundedTime, rhs.roundedTime).append(this.component,
				rhs.component).append(this.petalsId, rhs.petalsId).append(
				this.type, rhs.type).append(this.time, rhs.time).append(
				this.endpoint, rhs.endpoint).append(this.content, rhs.content)
				.append(this.status, rhs.status).append(
						this.messageExchangeSnapshotId,
						rhs.messageExchangeSnapshotId).append(this.operation,
						rhs.operation).append(this.service, rhs.service)
				.append(this.role, rhs.role).append(this.server, rhs.server)
				.isEquals();
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(-1930082573, 1461837515).appendSuper(
				super.hashCode()).append(this.roundedTime).append(
				this.component).append(this.petalsId).append(this.type).append(
				this.time).append(this.endpoint).append(this.content).append(
				this.status).append(this.messageExchangeSnapshotId).append(
				this.operation).append(this.service).append(this.role).append(
				this.server).toHashCode();
	}

	public String getMep() {
		return mep;
	}

	public void setMep(String mep) {
		this.mep = mep;
	}

}
