/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.client;

import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientInitializationException;
import org.ow2.petals.tools.webadmin.datacollector.util.StringHelper;

/**
 * @author mcarpentier A {@link PetalsJMXClientBean} stores details about a JMX
 *         connection
 * 
 */
public class PetalsJMXClientBean {

    /** client JMX settings * */
    private final String petalsJMXServer;

    private final int petalsJMXport;

    private final String petalsJMXPassword;

    private final String petalsJMXLogin;

    /**
     * @param petalsJMXServer
     * @param petalsJMXPorttalsJMXPort
     * @param petalsJMXPassword
     * @param petalsJMXLogin
     * @throws DataCollectorClientInitializationException
     * @throws DataCollectorClientException
     */
    public PetalsJMXClientBean(final String petalsJMXServer, final String petalsJMXPort,
            final String petalsJMXPassword, final String petalsJMXLogin)
            throws DataCollectorClientInitializationException, DataCollectorClientException {
        if (StringHelper.isNullOrEmpty(petalsJMXServer)) {
            throw new DataCollectorClientInitializationException(
                    "You must specify an host to connect to the petals jmx server");
        }
        if (StringHelper.isNullOrEmpty(petalsJMXPort)) {
            throw new DataCollectorClientInitializationException(
                    "You must specify a port to connect to the petals jmx server");
        }
        this.petalsJMXServer = petalsJMXServer;
        this.petalsJMXport = Integer.parseInt(petalsJMXPort);
        this.petalsJMXLogin = petalsJMXLogin;
        this.petalsJMXPassword = petalsJMXPassword;

    }

    public String getPetalsJMXLogin() {
        return this.petalsJMXLogin;
    }

    public String getPetalsJMXPassword() {
        return this.petalsJMXPassword;
    }

    public int getPetalsJMXport() {
        return this.petalsJMXport;
    }

    public String getPetalsJMXServer() {
        return this.petalsJMXServer;
    }

}
