/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.dao.message;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.jbi.messaging.MessageExchange;

import org.hibernate.Session;
import org.hibernate.Transaction;
import org.ow2.petals.tools.webadmin.datacollector.bo.message.MessageExchangeSnapshot;
import org.ow2.petals.tools.webadmin.datacollector.dao.DAOException;
import org.ow2.petals.tools.webadmin.datacollector.dao.GenericDAO;

/**
 * @author mcarpentier
 * 
 */
public class MessageExchangeSnapshotDAO extends GenericDAO {

    private static MessageExchangeSnapshotDAO theInstance;

    private MessageExchangeSnapshotDAO() {

    }

    public static synchronized MessageExchangeSnapshotDAO getInstance() {
        if (null == MessageExchangeSnapshotDAO.theInstance) {
            MessageExchangeSnapshotDAO.theInstance = new MessageExchangeSnapshotDAO();
        }
        return MessageExchangeSnapshotDAO.theInstance;
    }

    /**
     * @param session
     *            the hibernate session
     * @param petalsMessageExchangeId
     *            the id of the ME in petals
     * @return the {@link MessageExchange} with the given petalsId
     * @throws DAOException
     */
    public List<MessageExchangeSnapshot> retrieveAll(Session session) throws DAOException {
        try {
            final String query = "from MessageExchangeSnapshot mes ORDER BY mes.petalsId";
            return session.createQuery(query).list();
        } catch (final Exception e) {
            throw new DAOException("Error retrieving all MessageExchangeSnapshots", e);
        }
    }

    /**
     * @param session
     *            the hibernate session
     * @param petalsMessageExchangeId
     *            the id of the ME in petals
     * @return the {@link MessageExchange} with the given petalsId
     * @throws DAOException
     */
    public List<MessageExchangeSnapshot> retrieveByPetalsId(Session session,
            String petalsMessageExchangeId) throws DAOException {
        try {
            final String query = "from MessageExchangeSnapshot mes where mes.petalsId = '"
                    + petalsMessageExchangeId + "' ORDER BY mes.petalsId";
            return session.createQuery(query).list();
        } catch (final Exception e) {
            throw new DAOException("Error retrieving MessageExchangeSnapshots using petalsId : "
                    + petalsMessageExchangeId, e);
        }
    }

    /**
     * @param session
     * @param server
     * @return
     * @throws DAOException
     */
    public List<MessageExchangeSnapshot> retrieveByServer(Session session, String server)
            throws DAOException {
        try {
            final String query = "from MessageExchangeSnapshot mes where mes.server = '" + server
                    + "' ORDER BY mes.petalsId";
            return session.createQuery(query).list();
        } catch (final Exception e) {
            throw new DAOException("Error retrieving MessageExchangeSnapshots using server : "
                    + server, e);
        }
    }

    /**
     * @param session
     * @param server
     * @param component
     * @return
     * @throws DAOException
     */
    public List<MessageExchangeSnapshot> retrieveByServerAndComponent(Session session,
            String server, String component) throws DAOException {
        try {
            final String query = "from MessageExchangeSnapshot mes where mes.server = '" + server
                    + "' and mes.component = '" + component + "' ORDER BY mes.petalsId";
            return session.createQuery(query).list();
        } catch (final Exception e) {
            throw new DAOException(
                    "Error retrieving MessageExchangeSnapshots using server,component : " + server
                            + "," + component, e);
        }
    }

    /**
     * @param session
     * @param server
     * @param component
     * @return
     * @throws DAOException
     */
    public List<MessageExchangeSnapshot> retrieveByServerAndComponentAndEndpoint(Session session,
            String server, String component, String endpoint) throws DAOException {
        try {
            final String query = "from MessageExchangeSnapshot mes where mes.server = '" + server
                    + "' and mes.component = '" + component + "' and mes.endpoint = '" + endpoint
                    + "' ORDER BY mes.petalsId";
            return session.createQuery(query).list();
        } catch (final Exception e) {
            throw new DAOException(
                    "Error retrieving MessageExchangeSnapshots using server,component,endpoint : "
                            + server + "," + component + "," + endpoint, e);
        }
    }

    public void deleteOlder(Session session, long time) throws DAOException {
        try {
            Transaction tx = session.beginTransaction();

            String hqlDelete = "delete MessageExchangeSnapshot m where m.time <= :oldTime";
            // or String hqlDelete = "delete Customer where name = :oldName";
            session.createQuery(hqlDelete).setString("oldTime", String.valueOf(time))
                    .executeUpdate();
            tx.commit();
        } catch (final Exception e) {
            throw new DAOException("Error deleting MessageExchangeSnapshots older than "
                    + new Date(time) + "(" + time + ")", e);
        }
    }

    public long getMessagesExchangesSnapshotCount(Session session) throws DAOException {
        try {
            String query = "select count(*) from MessageExchangeSnapshot";
            return ((Long) session.createQuery(query).iterate().next()).longValue();
        } catch (final Exception e) {
            throw new DAOException("Can't get MessageExchangesSnapshot count", e);
        }
    }

    public long getMessagesExchangesCount(Session session) throws DAOException {
        try {
            long result = 0;
            String query = "select count(distinct m.petalsId) from MessageExchangeSnapshot m";
            Iterator it = session.createQuery(query).iterate();
            if (it != null) {
                result = ((Long) session.createQuery(query).iterate().next()).longValue();
            }
            return result;
        } catch (final Exception e) {
            throw new DAOException("Can't get MessageExchangesSnapshot count", e);
        }
    }
}
