/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.management.Notification;
import javax.management.NotificationListener;

import org.apache.log4j.Logger;
import org.hibernate.Session;
import org.ow2.petals.tools.webadmin.datacollector.bo.message.MessageExchangeSnapshot;
import org.ow2.petals.tools.webadmin.datacollector.dao.DAOException;
import org.ow2.petals.tools.webadmin.datacollector.dao.message.MessageExchangeSnapshotDAO;

/**
 * @author mcarpentier
 * 
 * Listnes for Messages Exchanges from petals , then save them
 * 
 */
public class MessagesListener implements NotificationListener {

    private final static Logger log = Logger.getLogger(MessagesListener.class);

    public MessagesListener() {
        super();
    }

    /**
     * @see NotificationListener#handleNotification(Notification, Object)
     */
    @SuppressWarnings("unchecked")
    public void handleNotification(Notification notification, Object handback) {
        Session session = null;
        try {
            session = HibernateHelper.getSession();
            final Object userData = notification.getUserData();
            if (userData instanceof ArrayList) {
                final List<Map<String, Object>> messages = (ArrayList<Map<String, Object>>) userData;
                for (final Map<String, Object> msg : messages) {
                    final MessageExchangeSnapshot m1 = new MessageExchangeSnapshot(msg);
                    MessageExchangeSnapshotDAO.getInstance().create(session, m1);
                }
            } else {
                MessagesListener.log.warn("Notification #" + notification.getSequenceNumber()
                        + " : userData aren't messages - datas instance of "
                        + userData.getClass().getName());
            }
            session.close();
        } catch (final DAOException e) {
            MessagesListener.log.error("Can't save message in database", e);
            if (session != null) {
                session.close();
            }
        }

    }
}
