/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: CollectorThread.java msauvage $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.util;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.ow2.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.dao.DAOException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;

/**
 * @author mcarpentier
 * 
 * This thread is used by the {@link CollectorServer} to try to reconnect to
 * PEtALS when connection is lost. Later, better support for multiple PEtALS
 * connections will be implemented
 * 
 */
public class ReconnectionThread extends Thread {

    private static Logger log = Logger.getLogger(ReconnectionThread.class);

    private int maxAttempts = 30;

    private int refreshFrequency = 10000;

    private int attemptCount;

    private final CollectorServer cs;

    private boolean isRunning = false;

    public ReconnectionThread(CollectorServer cs) {
        this.cs = cs;
        this.attemptCount = 0;
        this.refreshFrequency = cs.getReconnectionInterval();
        this.attemptCount = cs.getReconnectionAttempts();
        this.setDaemon(true);
    }

    @Override
    public void run() {
        this.isRunning = true;
        while (this.isRunning) {
            this.attemptCount++;
            if (this.attemptCount >= maxAttempts) {
                this.isRunning = false;
            }
            try {
                cs.setupServer();
                log.info("PEtALS connection re-established!");
                this.isRunning = false;
            } catch (DataCollectorServerException e) {
                /** don't need the stack trace**/
                log.info("Can't reconnect to PEtALS (" + (maxAttempts - attemptCount)
                        + " attempts left)");
            }
            try {
                synchronized (this) {
                    this.wait(this.refreshFrequency);
                }
            } catch (final InterruptedException ie) {
                this.isRunning = false;
            }
        }
    }

    public int getMaxAttempts() {
        return maxAttempts;
    }

    public void setMaxAttempts(int maxAttempts) {
        this.maxAttempts = maxAttempts;
    }

    public int getRefreshFrequency() {
        return refreshFrequency;
    }

    public void setRefreshFrequency(int refreshFrequency) {
        this.refreshFrequency = refreshFrequency;
    }

}
